<?php

//  Copyright (C) 2008 Meertens Instituut / KNAW
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

error_reporting(E_ALL);
require('Console/Getopt.php');
require('DB.php');
require('praat2sql_config.inc.php');
PEAR::setErrorHandling(PEAR_ERROR_DIE);

$tokenize_sentences = TRUE;

$args = Console_Getopt::getopt($argv, PRAAT_COMMANDLINE_OPTIONS);
$options = praat_parseargsarray($args[0]);

if (array_key_exists('h', $options) || empty($options)) {
	die(PRAAT_COMMANDLINE_HELP);
}

if (array_key_exists('t', $options)) {
	$tokenize_sentences = FALSE;
}

$database = $options['d'];

if (empty($args[1])) {
	die("Error: no filename given\n" . PRAAT_COMMANDLINE_HELP);
}
$input = realpath($args[1][0]);

$db_obj = DB::connect(PRAAT_MYSQL_EXTENSION . '://' . PRAAT_MYSQL_USER . ':' . PRAAT_MYSQL_PASSWORD . '@' . PRAAT_MYSQL_HOST . '/' . $database);
if (PEAR::isError($db_obj)) {
	die($db_obj->getMessage() . "\n" . $db_obj->getUserInfo() . "\n\n");
}

$db_obj->query('SET NAMES utf8');

if (array_key_exists('e', $options)) {
	praat_emptydatabase($db_obj);
}

if (is_dir($input)) {
	$inputfiles = praat_getinputfiles($input);
} elseif (is_file($input)) {
	$inputfiles = array($input);
}

foreach($inputfiles as $inputfile) {

	$fh = @ fopen($inputfile, 'r');
	
	if (! praat_checkpraatfile($fh)) {
		echo 'Either ' . basename($inputfile) . " does not exist, is not a PRAATfile or it is not encoded as UTF-8 ... skipped\n";
		continue;
	}
	
	list($xmin, $xmax) = praat_gettranscriptinfo($fh);
	$filename = basename($inputfile);
	
	$file_id = praat_savefileinfo($xmin, $xmax, $filename, $db_obj);
	
	list($tiernr, $tiername) = praat_getfirsttier($fh);
	$tier_id = praat_savetierinfo($file_id, $tiernr, $tiername, $db_obj);
	
	// initially FALSE
	$multilinetext = FALSE;
	
	while (! feof($fh)) {
		
		$line = fgets($fh);
		if (preg_match('/item \[(\d+)\]/', $line, $matches)) {
			$tiernr = $matches[1];
			$tiername = '';
		}
		if (preg_match('/name = "(.+?)"/', $line, $matches)) {
			$tiername =  $matches[1];
			$tier_id = praat_savetierinfo($file_id, $tiernr, $tiername, $db_obj);
		}
		
		// find next interval
		if (preg_match('/intervals \[(\d+)\]:/', $line, $matches)) {
			$intervalnr = $matches[1];
		}
		
		if (preg_match('/xmin = ([\d|.]+)/', $line, $matches)) {
			$xmin = $matches[1];
		}
		
		if (preg_match('/xmax = ([\d|.]+)/', $line, $matches)) {
			$xmax = $matches[1];
		}
	
		// read text on single line
		if (preg_match('/text = "(.+?)"/', $line, $matches)) {
			$multilinetext = FALSE;
			$text = trim($matches[1]);
			$interval_id = praat_savesentenceinfo($tier_id, $intervalnr, $xmin, $xmax, $text, $db_obj);
			if ($tokenize_sentences) {
				praat_savewordinfo($interval_id, $text, $db_obj);
			}
		}
	
		// read text on multiple lines
		if (preg_match('/text = "([^"]+)$/', $line, $matches)) {
			$multilinetext = TRUE;
			$text = trim($matches[1]);
		} elseif ($multilinetext && strpos($line, '"') === FALSE) {
			$text .= ' ' . trim($line);
		} elseif ($multilinetext) {
			$chars_to_trim = '"' . " \t\n\r\0\X0B";
			$text .= ' ' . trim($line, $chars_to_trim);
			$multilinetext = FALSE;
			$interval_id = praat_savesentenceinfo($tier_id, $intervalnr, $xmin, $xmax, $text, $db_obj);
			if ($tokenize_sentences) {
				praat_savewordinfo($interval_id, $text, $db_obj);
			}
		}
	
	}
	
	fclose($fh);
	
	echo  basename($inputfile) . " successfully processed\n";
}

function praat_checkpraatfile(&$fh) {

	if (! $fh) {
		return FALSE;
	}
	
	fgets($fh);  // first line of the file, ignored
	$line = trim(fgets($fh));  // second line of the file

	// second line of the file should contain this
	if ($line != 'Object class = "TextGrid"') {
		return FALSE;
	} else {
		return TRUE;
	}
}

function praat_getinputfiles($directory) {

	$files = array();
	
	try {
		$dir  = new DirectoryIterator($directory);
		foreach ($dir as $file) {
			if ($file->isDot() || $file->isDir()) {
				continue;
			}
			$files[] = $file->getPath() . DIRECTORY_SEPARATOR . $file;
		}
	}
	
	catch(Exception $e) {
		echo $e->getMessage() . "\n";
	}

	return $files;
}

function praat_gettranscriptinfo(&$fh) {

	fgets($fh); // empty line
	$xmin = trim(fgets($fh));
	$xmax = trim(fgets($fh));
	
	preg_match('/xmin = ([\d|.]+)/', $xmin, $matches);
	$xmin = $matches[1];
	preg_match('/xmax = ([\d|.]+)/', $xmax, $matches);
	$xmax = $matches[1];
	
	return array($xmin, $xmax);
}

function praat_getfirsttier(&$fh) {

	// find start of tier
	do { $line = fgets($fh); } while (! (preg_match('/item \[(\d+)\]/', $line, $matches) || feof($fh)));

	if (isset($matches[1])) {
		$tiernr = $matches[1];
	}

	// find name of tier
	do { $line = fgets($fh); } while (! (preg_match('/name = "(.+?)"/', $line, $matches) || feof($fh)));
	
	if (isset($matches[1])) {
		$tiername = $matches[1];
	}

	
	return array($tiernr, $tiername);
}

function praat_savefileinfo($xmin, $xmax, $filename, $db_obj) {

	$db_obj->query(PRAAT_INSERT_TRANSCRIPT_QUERY, array($xmin, $xmax, $filename));
	$file_id = $db_obj->getOne('SELECT LAST_INSERT_ID()');	
	
	return $file_id;
}

function praat_savetierinfo($file_id, $tiernr, $tiername, $db_obj) {

	$db_obj->query(PRAAT_INSERT_TIER_QUERY, array($file_id, $tiernr, $tiername));
	$tier_id = $db_obj->getOne('SELECT LAST_INSERT_ID()');	
	
	return $tier_id;
}

function praat_savesentenceinfo($tier_id, $intervalnr, $xmin, $xmax, $text, $db_obj) {

	$db_obj->query(PRAAT_INSERT_SENTENCE_QUERY, array($tier_id, $intervalnr, $xmin, $xmax, $text));
	$interval_id = $db_obj->getOne('SELECT LAST_INSERT_ID()');	
	
	return $interval_id;
}

function praat_savewordinfo($interval_id, $text, $db_obj) {
	
	$words = preg_split(PRAAT_SPLIT_SENTENCE_REGEXP, $text, -1, PREG_SPLIT_NO_EMPTY);
	foreach($words as $index => $word) {
		if (PRAAT_WORDS_TO_LOWERCASE) {
			$word = mb_strtolower($word, 'UTF-8');
		}
		$db_obj->query(PRAAT_INSERT_WORD_QUERY, array($interval_id, $word, $index + 1));
	}
}

function praat_parseargsarray($args) {

	$options = array();
	foreach($args as $option) {
		$options[$option[0]] = $option[1];
	}
	
	return $options;
}

function praat_emptydatabase($db_obj) {

	// order is important when using InnoDB tables
	$db_obj->query(PRAAT_EMPTY_WORD_QUERY);
	$db_obj->query(PRAAT_EMPTY_SENTENCE_QUERY);
	$db_obj->query(PRAAT_EMPTY_TIER_QUERY);
	$db_obj->query(PRAAT_EMPTY_TRANSCRIPT_QUERY);
}

?>
